# Z39.50 values according to https://www.loc.gov/z3950/agency/bib1.html
# and NLI info according to http://web.nli.org.il/sites/NLI/Hebrew/infochannels/librarians/Pages/Z39.50.aspx 
# Name of Database: NNL01
# Host name: aleph.nli.org.il
# IP address: 192.114.7.200
# Port: 9991
# Character-set: UTF-8
# We support the following record syntaxes:
# USMARC, OPAC, XML
# We support the following word searches:
# 1016, 1017, 1,1003, 1004,4,21,30,31,7,12,1007,1031,1007,5028,1033
# We support the following phrase searches:
# 7,12,1,1003,1004,4,21,15
# We support the following sorts:
# 1,4,30,31,1003

module Gared
  require 'zoom'
  require 'nokogiri'
  class Aleph
    def initialize(host, port, database, syntax = 'USMARC')
      @options = {host: host, port: port, database: database, syntax: syntax}
    end
    def query_persons(q)
    end

    def query_person(person)
    end

    def query_publications(q)
    end

    def query_publication(publication)
    end

    def query_publications_by_person(person, ctx = nil)
      ZOOM::Connection.open(@options[:host], @options[:port]) do |conn|
        conn.database_name = @options[:database] # 'aleph.nli.org.il',9991
        conn.preferred_record_syntax = @options[:syntax]
        rset = conn.search("@attr 1=1003 @attr 2=3 @attr 4=1 \"#{person}\"")
        rr = rset.records
        return nil if rr.nil? or rr.empty?
        ret = []
        rr.each do |r|
          xml = Nokogiri::Slop(r.xml)
          xml.remove_namespaces! # keeps biting me :)
          # these scrapes are based on the National Library of Israel usage. No attempt to make it generic. :)
          p = Publication.new(ctx)
          begin
            p.author_line = xml.xpath('//datafield[@tag=\'100\']/subfield[@code=\'a\']')[0].text
            # puts "author: #{p.author_line}" # DEBUG
          rescue
            nil
          end
          begin
            p.title = xml.xpath('//datafield[@tag=\'245\']/subfield[@code=\'a\']')[0].text
            # puts "title: #{p.title}" # DEBUG
          rescue
            nil
          end
          begin
            p.notes = xml.xpath('//datafield[@tag=\'500\']/subfield[@code=\'a\']').collect{|note| note.text}.join("\n")
          rescue
            nil
          end
          begin
            h = Holding.new
            h.source_id = xml.xpath('//datafield[@tag=\'090\']/subfield[@code=\'a\']')[0].text
            h.source_name = @options[:database]
            p.add_holding(h)
            ret << p
          rescue
            nil # ignore records with no holdings; they may be archival files or other non-publications
          end # 
        end
        return ret
      end
    end
  end
end